/**
 * Background class
 * 
 * Creates a dynamic cyberpunk background with subtle animations
 */
class Background {
  // Background properties
  color bgColor;
  color accentColor1;
  color accentColor2;
  
  // Movement properties
  float noiseScale = 0.02;
  float noiseOffset = 0;
  float noiseSpeed = 0.005;
  
  // Timing
  float cps = 0.5;
  float lastCycleTime = 0;
  
  Background() {
    // Dark cyberpunk colors
    bgColor = color(10, 12, 18);
    accentColor1 = color(0, 70, 100);
    accentColor2 = color(60, 0, 80);
  }
  
  void update() {
    // Update noise movement
    noiseOffset += noiseSpeed;
    
    // Check for cycle changes
    float currentCycleTime = millis() / 1000.0 * cps;
    if (floor(currentCycleTime) > floor(lastCycleTime)) {
      // New cycle - could trigger effects here
    }
    lastCycleTime = currentCycleTime;
  }
  
  void display() {
    // Create gradient background
    noiseDetail(8, 0.5);
    
    // Fill with base color
    noStroke();
    fill(bgColor, 40); // Semi-transparent for motion blur effect
    rect(0, 0, width, height);
    
    // Add subtle noise pattern
    float motionFactor = sin(millis() * 0.001) * 0.5 + 0.5;
    
    loadPixels();
    for (int y = 0; y < height; y += 4) {
      for (int x = 0; x < width; x += 4) {
        float noiseVal = noise(x * noiseScale, y * noiseScale, noiseOffset);
        
        if (noiseVal > 0.7) {
          color pixelColor;
          
          // Create different zones
          if (noiseVal > 0.85) {
            // Highlight areas
            pixelColor = lerpColor(accentColor1, accentColor2, 
                                   sin(x * 0.01 + millis() * 0.0005) * 0.5 + 0.5);
            pixelColor = color(red(pixelColor), green(pixelColor), blue(pixelColor), 
                              20 + 20 * motionFactor);
          } else {
            // Subtle accent
            pixelColor = lerpColor(bgColor, accentColor1, 0.3);
            pixelColor = color(red(pixelColor), green(pixelColor), blue(pixelColor), 10);
          }
          
          // Draw 4x4 pixel block for better performance
          fill(pixelColor);
          rect(x, y, 4, 4);
        }
      }
    }
    
    // Draw horizontal scan lines
    drawScanlines();
  }
  
  void drawScanlines() {
    stroke(255, 8);
    strokeWeight(1);
    for (int y = 0; y < height; y += 4) {
      line(0, y, width, y);
    }
    
    // Draw brighter scanline that moves
    float movingScanline = (millis() % 5000) / 5000.0 * height;
    stroke(255, 15);
    strokeWeight(2);
    line(0, movingScanline, width, movingScanline);
  }
  
  void setCPS(float newCps) {
    this.cps = newCps;
  }
}
